from click import group

from .cli_commands.init.init import init
from .cli_commands.commit.commit import commit
from .cli_commands.publish.publish import publish

@group()
def cli():
    pass

cli.add_command(init)
cli.add_command(commit)
cli.add_command(publish)

if __name__ == "__main__":
    cli()



from json import load, dump

def update_version(version: str) -> None:
    with open('dgupdaterconf.json') as f:
        dgupdaterconf_json = load(f)
    
    dgupdaterconf_json['version'] = version

    with open('dgupdaterconf.json', 'w') as f:
        dump(dgupdaterconf_json, f, indent = 4)



from os import makedirs, getcwd
from os.path import join
from json import load, dump, dumps
from shutil import rmtree

from .find_files import find_files
from .create_chunks import create_chunks

def commit_changes() -> None:

    try:
        rmtree('dgupdater_release/chunks') # Removing the chunks directory and its contents if it exists
    except FileNotFoundError as _:
        pass

    makedirs('dgupdater_release/chunks', exist_ok = True)

    release_json = {}

    release_files = find_files()
    # print(release_files)

    for file in release_files:

        file_path = f'{getcwd()}{file}'
        try:
            with open(file_path, 'r') as f:
                # print(file_path)
                release_json[file] = f.read()
        except UnicodeDecodeError as _:
            with open(file_path, 'rb') as f:  
                content = str(f.read())
                release_json[file] = content
                # print(content[:10])

    release_json_str = dumps(release_json)

    with open('dgupdaterconf.json') as f:
        dgupdaterconf_json = load(f)

    no_of_chunks = create_chunks(release_json_str, dgupdaterconf_json['app_name']) #creating chunks as well as getting the number of chunks
    
    dgupdaterconf_json['no_of_files'] = len(release_files)
    dgupdaterconf_json['update_ready'] = False
    dgupdaterconf_json['no_of_chunks'] = no_of_chunks
    dgupdaterconf_json['files_in_latest_version'] = release_files

    with open(join(getcwd(), 'dgupdater_release', 'dgupdaterconf.json'), 'w') as f:
        dump(dgupdaterconf_json, f, indent = 4)

if __name__ == "__main__":
    commit_changes()


from click import command, option, prompt, echo

from .func.check_configuration_files_exists import check_configuration_files_exists
from .func.update_version import update_version
from .func.commit_changes import commit_changes
from ..publish.func.publish_changes import publish_changes

@command()
@option('--version', '-v', required = True, prompt = 'New version number', help='New version number of the application')
def commit(version: str) -> None:   

    check_configuration_files_exists()
    
    if prompt("Are you sure you want to commit the changes? (y/n)").lower() in ["y", "yes"]:
        echo("Committing...")

        update_version(version)
        commit_changes()
        
        echo("Committed successfully.\nChanges are ready to be published.\n")    

        if prompt("Do you want to publish the changes now? (y/n)").lower() in ["y", "yes"]:
            publish_changes()
        else:
            echo("You can publish the changes later by running 'dgupdater publish' command.")
        return
    
    echo("Commit aborted.")




from click import command, echo, option

from .func.check_mongo_string import check_mongo_string
from .func.check_app_exists import check_app_exists
from .func.create_entry_in_mongodb import create_entry_in_mongodb
from .func.create_configuration_files import create_configuration_files

arguments = {
    'name': {
        'prompt': "Name of the Application",
        'help': "The name of the Application. It should be unique for every app "
    },
    'version': {
        'prompt': "Version of the Application",
        'help': "The version of the Application."
    },
    'mongodb_connection_string_write': {
        'prompt': "MongoDB Connection String with write access",
        'help': "The MongoDB Connection String for a cloud cluster, to be used by the developer for releasing update."
    },
    'mongodb_connection_string_read': {
        'prompt': "MongoDB Connection String with read access",
        'help': "The MongoDB Connection String for a cloud cluster, to be used by the clients for updating their applications."
    }    
}

#template
dgupdaterconf_json = {
    "obj_type": "config",
    "_id": "",
    "app_name": "",
    "version": "",
    "mongodb_connection_string_client": "",
    "files_in_latest_version": []
}


@command()
@option("--name", "-n", required = True, prompt = arguments["name"]["prompt"], help = arguments["name"]["help"])  
@option("--mongodbstrd", "-md", callback = check_mongo_string, required = True, prompt = arguments["mongodb_connection_string_write"]["prompt"], help = arguments["mongodb_connection_string_write"]["help"])
@option("--mongodbstrc", "-mc", callback = check_mongo_string, required = True, prompt = arguments["mongodb_connection_string_read"]["prompt"], help = arguments["mongodb_connection_string_read"]["help"])
def init(name: str, mongodbstrd: str, mongodbstrc: str) -> None:
    # print(name, version, mongodbstrd, mongodbstrc)    

    echo("\nInitializing this directory for autoupdation...")

    app_exists, over_write = check_app_exists(name, mongodbstrd)

    if app_exists and not over_write:
        echo("Initialization Aborted.")
        return
    elif app_exists and over_write:
        echo("Overwriting the existing application.")    
    
    dgupdaterconf_json["_id"] = name + '_config'
    dgupdaterconf_json["app_name"] = name
    dgupdaterconf_json["version"] = "Version will be updated after publishing the changes."
    dgupdaterconf_json["mongodb_connection_string_client"] = mongodbstrc
    dgupdaterconf_json["files_in_latest_version"] = 'Files will be listed after publishing the changes.'

    create_entry_in_mongodb(dgupdaterconf_json, mongodbstrd, over_write)

    create_configuration_files(dgupdaterconf_json, name, mongodbstrd)

    echo("Initialization Successful.")    
        
if __name__ == "__main__":
    init()
    


from pymongo import MongoClient
from click import echo, prompt

def check_app_exists(name: str, mongostr: str) -> bool:
    
    try:
        with MongoClient(mongostr) as client:
            db = client['DGUPDATER']
            collections = [collection.lower() for collection in db.list_collection_names()]
    except Exception as _:
        echo("Some error occured. Try again.")

    if name.lower() in collections:
        echo("\nApplication with the same name already exists in Mongodb.\n")

        if prompt("Do you want to overwrite the existing application? (y/n): ").lower() in ["y", "yes" ]:
            return True, True # App exists and overwrite
        return True, False # App exists but don't overwrite (exit)
    
    return False, False # App doesn't exist
    



from os import walk, getcwd
from os.path import normpath


def find_files() -> list:
    
    files_ = []
    retrieved_files = []

    ignored_list = get_ignore_list()
    ignored_list.append('dgupdater_release')

    for root, dirs, files in walk(getcwd()):

        # start: Ignore files logic
        ignore = False
        for ignored in ignored_list:
            if ignored in root:
                ignore = True
        if ignore:
            ignore = False
            continue
        # end: Ignore files logic

        dirs = [dir for dir in dirs if dir not in ignored_list]
        files = [file for file in files if file not in ignored_list]
     
        files_.append((normpath(root), dirs, files))

    for files in files_:
        for file in files[2]:
            file = files[0].split(getcwd())[1] + '\\' + file
            retrieved_files.append(file)   
    

    return retrieved_files


def get_ignore_list() -> list:
    try:
        with open('.dgupdaterignore', 'r') as file:
            return [line.split('\n')[0] for line in file.readlines()]
    except FileNotFoundError as _:
        return []

if __name__ == "__main__":
    print(get_ignore_list())
    find_files()
    


    # for file in find_files():
    #     print(file)

    # print(normpath('D:\\PROGRAMMING\\PROJECTS\\dgupdater'))

    # print(getsize('D:\PROGRAMMING\PROJECTS\dgupdater\dgupdaterconf.json'))
    # print(getsize('ashif'))


        
        


from json import dump, load
from json.decoder import JSONDecodeError
from os import makedirs
from os.path import join, exists
from platformdirs import user_data_dir

def create_configuration_files(data: dict, app_name: str, mongodbstrd: str) -> None:
    with open("dgupdaterconf.json", "w") as f:
        dump(data, f, indent = 4)

    dgupdater_dir = user_data_dir("dgupdater", "DarkGlance")
    makedirs(dgupdater_dir, exist_ok = True)

    dgupdaterconf_json = {
        'mongodbstrds': {}
    }

    file = join(dgupdater_dir, "dgupdaterconf.json")
    
    try:
        if exists(file):
            with open(file, 'r') as f:
                dgupdaterconf_json = load(f)
    except JSONDecodeError as _:
        pass

    dgupdaterconf_json['mongodbstrds'][app_name] = mongodbstrd    
    
    with open(file, "w") as f:
        dump(dgupdaterconf_json, f, indent = 4)

    with open('.dgupdaterignore', 'w') as f:
        f.write('')



if __name__ == "__main__":
    create_configuration_files({}, 'ashif')



from pymongo import MongoClient
from click import UsageError

def create_entry_in_mongodb(dgupdaterconf_json: dict[str:any], mongodbstrd: str, over_write: bool) -> None:
    try:
        with MongoClient(mongodbstrd) as client:

            db = client['DGUPDATER']
            collection = db[dgupdaterconf_json["app_name"]]

            if over_write:
                collection.delete_one({"_id": dgupdaterconf_json["_id"]})
                

            collection.insert_one(dgupdaterconf_json)
    except Exception as _:
        raise UsageError("Error while creating entry in MongoDB: ")


if __name__ == "__main__":
    raise UsageError("This file is not meant to be executed directly.")


from json import dump
from math import ceil
from tqdm import tqdm
from os.path import join
from os import getcwd


def create_chunks(release_json_str: str, app_name: str) -> int:
    
    max_chunk_size = 1_000_000 # 10 Lakh (or) 1 Million

    release_json_size = len(release_json_str)

    no_of_chunks = ceil(release_json_size / max_chunk_size)
    
    start = 0
    
    with tqdm(total = no_of_chunks, desc = "Creating Chunks", ncols = 110, unit='chunks') as pbar: # progres bar
        
        for i in range(no_of_chunks):
            chunk_part = i + 1
            chunk_name = f'{app_name}_part{chunk_part}'

            chunk = release_json_str[start:start + max_chunk_size]        
            chunk = {
                'obj_type': 'chunk',
                '_id': chunk_name,
                'chunk_data': chunk
            }

            with open(join(getcwd(), 'dgupdater_release', 'chunks', f'{chunk_name}.json'), 'w') as f:
                dump(chunk, f, indent = 4)

            start += max_chunk_size
            pbar.update(1)

    return no_of_chunks







from click import command, echo, prompt

from .func.check_release_files_exists import check_release_files_exists
from .func.publish_changes import publish_changes


@command()
def publish() -> None:

    check_release_files_exists()

    if prompt("Are you sure you want to publish the changes? (y/n)").lower() in ["y", "yes"]:

        publish_changes()

        return
    
    echo("Aborted publishing changes.")


    



        
    


from setuptools import setup, find_packages
# print(find_packages())

setup(
    name="dgupdater",
    version="0.3.2",
    author="DarkGlance",
    author_email="darkglance.developer@gmail.com",
    description="A CLI based auto updation assistant tool for python applications",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 2 - Pre-Alpha',
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Topic :: Utilities'
    ],
    python_requires='>=3.6',
    install_requires=[
        "click>=7.0",
        "pymongo",
        "platformdirs"

    ],
    entry_points={
        'console_scripts': [
            'dgupdater=dgupdater:cli',
        ],
    },
    project_urls = {
        'Github': "https://github.com/Ashif4354/dgupdater"
    },
    keywords= [
        'dgupdater', 'auto updater', 
        'cli tool', 'python', 
        'python application', 'auto updation', 
        'auto update', 'update',
        'cli', 'command line interface'
        'no code', 'low code', 'no code tool',
        'mongodb', 'pymongo', 'click', 
        'darkglance', 'darkglance developer',
        'ashif', 'pypi', 'pip',
    ]
)



from os.path import exists
from click import echo, UsageError
from json import load

from ...init.init import dgupdaterconf_json as template_dgupdaterconf_json

def check_configuration_files_exists() -> None:
    if not exists('dgupdaterconf.json'):
        raise UsageError("'dgupdaterconf.json' file not found. Run 'dgupdater init' first. Also check if you are in the correct directory.")
    
    with open('dgupdaterconf.json') as f:
        dgupdaterconf_json = load(f)

    if (dgupdaterconf_json.keys() != template_dgupdaterconf_json.keys()):
        raise UsageError("'dgupdaterconf.json' file seems to be corrupted. Try 'dgupdater init' again.")






from pymongo import MongoClient
from pymongo.errors import ConnectionFailure
from click import BadParameter, UsageError, echo
from json import loads

def check_mongo_string(ctx, param, value: str) -> str|None:

    # start: detecting second execution of callback of click.options
    if value.endswith('*VERIFIEDONCE*'):
        return value[:-14]    
    # end: detecting second execution of callback of click.options

    echo('Verifying MongoDB Connection String...')

    if not value.startswith('mongodb'):
        raise BadParameter("Enter a Valid MongoDB Connection String")

    try:
        client = MongoClient(value, serverSelectionTimeoutMS = 3000)
        client.admin.command('ping')
        
        echo("Verified !!\n")
        return value + '*VERIFIEDONCE*'  # *VERIFIEDONCE* is added to the end of the string to detect second execution of callback of click.options()
    except ConnectionFailure as _:
        echo()
        raise BadParameter("Enter a Valid MongoDB Connection String")
    except Exception as e:
        raise UsageError("Some error occured. Try again." + str(e))
        
    

if __name__ == "__main__":
    print(check_mongo_string(1, 1, 'a'))